/*-----------------------------------------------------------------------------
 * EcDeviceDummy.cpp       
 * Copyright                acontis technologies GmbH, Weingarten, Germany
 * Description              Dummy Link Layer driver for demo 
 *---------------------------------------------------------------------------*/

/*-LOGGING-------------------------------------------------------------------*/
#ifndef pEcLogParms
#define pEcLogParms           (pEcLogParmsLL)
#endif
#define pEcLogParmsLLDefault  (pEcLogParmsLL)

/*-INCLUDES------------------------------------------------------------------*/
#include "EcDeviceDummy.h"
#include "EcError.h"
#include "EcType.h"
#include "EcLink.h"
#include "EcCommon.h"

/*-DEFINES-------------------------------------------------------------------*/

/***************************************************************************************************
*                                 FORWARD DECLARATIONS
*/

static EC_T_VOID EcLinkDebugMsg(
    EC_T_PVOID pInstance,
    EC_T_BYTE byEthTyp0, 
    EC_T_BYTE byEthTyp1, 
    const EC_T_CHAR* szFormat, ...
    );
                                                  
static EC_T_LINKSTATUS EcLinkGetStatus(EC_T_VOID* pvInstance);
static EC_T_DWORD EcLinkClose(EC_T_VOID* pvInstance);
static EC_T_VOID  EcLinkFreeSendFrame(EC_T_VOID* pvInstance, EC_T_LINK_FRAMEDESC* pLinkFrameDesc);

/***************************************************************************************************
*                                 GLOBALS
*/

/***************************************************************************************************
*                                 LOCALS
*/
static PT_DUMMY_INTERNAL    S_oOpenInstanceRoot  = EC_NULL;
static EC_T_INT             S_nOpenedInstances   = 0;

/***************************************************************************************************
*                                 HELPER FUNCTIONS
*/

/** List management */

/*****************************************************************************
* Adds an adapter to the list of configured adapters
*/
static EC_T_BOOL ListAddOI(PT_DUMMY_INTERNAL poAdapter)
{
    EC_T_BOOL           bResult = EC_TRUE;
    PT_DUMMY_INTERNAL   oCur = EC_NULL;
    
    oCur = S_oOpenInstanceRoot;
    
    if (EC_NULL == oCur)
    {
        S_oOpenInstanceRoot = poAdapter;
        S_oOpenInstanceRoot->pPrev = S_oOpenInstanceRoot->pNext = EC_NULL;
    }
    else
    {
        while (EC_NULL != oCur->pNext)
        {
            oCur = oCur->pNext;
        }
        
        oCur->pNext = poAdapter;
        poAdapter->pPrev = oCur;
        poAdapter->pNext = EC_NULL;
    };
    
    return bResult;
}
    
/*****************************************************************************
/* Removes an adapter from the list of configured adapters
*/
static EC_T_BOOL ListRmOI(PT_DUMMY_INTERNAL poAdapter)
{
    EC_T_BOOL       bResult = EC_TRUE;
    
    if (S_oOpenInstanceRoot == poAdapter)
    {
        S_oOpenInstanceRoot = poAdapter->pNext;
    }
    
    if (EC_NULL != poAdapter->pPrev)
    {
        poAdapter->pPrev->pNext = poAdapter->pNext;
    }
    
    if (EC_NULL != poAdapter->pNext)
    {
        poAdapter->pNext->pPrev = poAdapter->pPrev;
    }
    
    poAdapter->pPrev = EC_NULL;
    poAdapter->pNext = EC_NULL;
    
    return bResult;
}
    
/*****************************************************************************
/* Looks up for an adapter in the list of configured adapters
*/
static PT_DUMMY_INTERNAL ListSeekOI(EC_T_DWORD dwBase)
{
    PT_DUMMY_INTERNAL  oCur = EC_NULL;
    
    oCur = S_oOpenInstanceRoot;
    
    while ((EC_NULL != oCur) && (dwBase != oCur->dwMemRegisterBase))
    {
        oCur = oCur->pNext;
    }
    
    return oCur;
}

/***************************************************************************************************/
/**
\brief  MAP BAR Registers.

\return EC_TRUE on success, EC_FALSE otherwise.
*/
static EC_T_BOOL MapMemory(PT_DUMMY_INTERNAL pAdapter)
{
    EC_T_BOOL           bRet    = EC_TRUE;
    
    EC_UNREFPARM(pAdapter);

    /* Map adapter memory here */
    /* <= change here */

    return bRet;
}

/***************************************************************************************************/
/**
\brief  UnMap / Free BARs.

\return EC_TRUE on success, EC_FALSE otherwise.
*/
static EC_T_BOOL UnmapMemory(PT_DUMMY_INTERNAL pAdapter)
{
    EC_T_BOOL   bRet = EC_TRUE;
    
    EC_UNREFPARM(pAdapter);

    /* Unmap adapter memory here */
    /* <= change here */

    return bRet;
}

/********************************************************************************/
/** \brief Open Link Layer connection.
*
* \return EC_E_NOERROR or error code.
*/
static EC_T_DWORD EcLinkOpen(
    EC_T_VOID*                  pvLinkParms,            /* [in]  link parameters */
    EC_T_RECEIVEFRAMECALLBACK   pfReceiveFrameCallback, /* [in]  pointer to rx callback function */
    EC_T_LINK_NOTIFY            pfLinkNotifyCallback,   /* [in]  pointer to notification callback function */
    EC_T_VOID*                  pvContext,              /* [in]  caller context, to be used in callback functions */
    EC_T_VOID**                 ppvInstance             /* [out] instance handle */
    )
{
#if (defined INCLUDE_LOG_MESSAGES)
#undef  pEcLogParmsLL
#define pEcLogParmsLL pEcLogParmsLLOpen
#endif
    EC_T_DWORD              dwRetVal = EC_E_ERROR;
    EC_T_LINK_PARMS_DUMMY*  pLinkParmsAdapter = (EC_T_LINK_PARMS_DUMMY*)pvLinkParms;
    EC_T_DWORD              dwMemRegisterBase = 0;
                                    
    PT_DUMMY_INTERNAL       pAdapter = EC_NULL;
    EC_T_VOID*              pDummyInstance    = EC_NULL;
       
    EC_UNREFPARM(pfLinkNotifyCallback);

    /* avoid NULL pointer access */
    if (ppvInstance == EC_NULL)
    {
        ppvInstance = &pDummyInstance;
    }

    /* check Signature */
    *ppvInstance = EC_NULL;

    if (EC_NULL == pLinkParmsAdapter)
    {
        EcLogMsg(EC_LOG_LEVEL_ERROR, (pEcLogContext, EC_LOG_LEVEL_ERROR, "Dummy-EcLinkOpen(): Missing Configuration for Dummy Link Layer\n"));
        *ppvInstance = EC_NULL;
        dwRetVal = EC_E_INVALIDPARM;
        goto Exit;
    }

    if (pLinkParmsAdapter->linkParms.dwSignature != EC_LINK_PARMS_SIGNATURE_DUMMY)
    {
        EcLogMsg(EC_LOG_LEVEL_ERROR, (pEcLogContext, EC_LOG_LEVEL_ERROR, "Dummy-EcLinkOpen(): Invalid Configuration for Dummy Link Layer\n"));
        *ppvInstance = EC_NULL;
        dwRetVal = EC_E_INVALIDPARM;
        goto Exit;
    }
    
    if (EC_NULL == ppvInstance)
    {
        EcLogMsg(EC_LOG_LEVEL_ERROR, (pEcLogContext, EC_LOG_LEVEL_ERROR, "Dummy-EcLinkOpen(): No Space for Driver Instance handle provided\n"));
        dwRetVal = EC_E_INVALIDPARM;
        goto Exit;
    }

    if (pLinkParmsAdapter->linkParms.dwInstance == 0)
    {
        /* the first unit was meant as a value of 0 is not defined! */ 	    	
        pLinkParmsAdapter->linkParms.dwInstance = 1;
    }

    /* Initialize PCI here (if needed) */
    /* <= change here */

    /* Check whether the instance is alraedy in use */
    if (EC_NULL != ListSeekOI(dwMemRegisterBase))
    {
		EcLogMsg(EC_LOG_LEVEL_ERROR, (pEcLogContext, EC_LOG_LEVEL_ERROR, "Dummy-EcLinkOpen(): Instance already in use!!\n"));
        dwRetVal = EC_E_INVALIDPARM;
        goto Exit;
    };
    
    /* Register receive callback function if needed */
    if ((EC_NULL == pfReceiveFrameCallback) && (EcLinkMode_INTERRUPT == pLinkParmsAdapter->linkParms.eLinkMode))
    {
        EcLogMsg(EC_LOG_LEVEL_ERROR, (pEcLogContext, EC_LOG_LEVEL_ERROR, "Dummy-EcLinkOpen(): Missing receive frame callback\n"));
        dwRetVal = EC_E_INVALIDPARM;
        goto Exit;
    }
    
    /* create new instance */
    pAdapter = (PT_DUMMY_INTERNAL)LinkOsMalloc(sizeof(T_DUMMY_INTERNAL));
    if (EC_NULL == pAdapter)
    {
        EcLogMsg(EC_LOG_LEVEL_ERROR, (pEcLogContext, EC_LOG_LEVEL_ERROR, "Dummy-EcLinkOpen(): Error allocating memory\n"));
        dwRetVal = EC_E_NOMEMORY;
        goto Exit;
    }
    LinkOsMemset(pAdapter, 0, sizeof(T_DUMMY_INTERNAL));
    LinkOsMemcpy(&(pAdapter->oInitParms), pLinkParmsAdapter, sizeof(EC_T_LINK_PARMS_DUMMY));
	pAdapter->dwSignature = EC_LINK_PARMS_SIGNATURE_DUMMY;

	/* create context */
    dwRetVal = LinkOsCreateContext((EC_T_VOID*)&pLinkParmsAdapter->linkParms, &pAdapter->pvLinkOsContext);
    if (EC_E_NOERROR != dwRetVal)
    {
        goto Exit;
    }

    /* do initialization here */
    if (!MapMemory(pAdapter))
    {
        EcLogMsg(EC_LOG_LEVEL_ERROR, (pEcLogContext, EC_LOG_LEVEL_ERROR, "Dummy-EcLinkOpen(): Error Mapping memory\n"));
        dwRetVal = EC_E_NOMEMORY;
        goto Exit;
    }

    /* Example, allocate internal memory */
    pAdapter->pbyIntBuffer = (EC_T_BYTE*)LinkOsMalloc(1536);
    pAdapter->dwFrameLen = 0;
    pAdapter->bNewData = EC_FALSE;
    if (EC_NULL == pAdapter->pbyIntBuffer)
    {
        EcLogMsg(EC_LOG_LEVEL_ERROR, (pEcLogContext, EC_LOG_LEVEL_ERROR, "Dummy-EcLinkOpen(): Error allocating memory\n"));
        dwRetVal = EC_E_NOMEMORY;
        goto Exit;
    }

    /* configure new instance */
    pAdapter->pfReceiveFrameCallback = pfReceiveFrameCallback;
    pAdapter->pvCallbackContext = pvContext;

    /* Interrupt handling */
    /* <= change here */

    /* Set up Device Control registers (incl. Device reset) */
    /* <= change here */
    
    /* Initialize DMA control */
    /* <= change here */

    /* now configure DMA space */
    
    /* STOP NIC RX & TX */
    /* <= change here */
    /*
    dwRes = XXXX_StopReceiver(pAdapter);
    if (EC_E_NOERROR != dwRes)
    {
        dwRetVal = dwRes;
        goto Exit;
    }
    dwRes = XXXX_StopTransceiver(pAdapter);
    if (EC_E_NOERROR != dwRes)
    {
        dwRetVal = dwRes;
        goto Exit;
    }
    */
    
    /* Initialize DMA Memory */
    /* <= change here */
    /*
    dwRes = XXXX_InitDmaMemory(pAdapter);
    if (EC_E_NOERROR != dwRes)
    {
        dwRetVal = dwRes;
        goto Exit;
    }
    */
    
    /* startup the receiver */
    /* <= change here */
    /*
    dwRes = XXXX_StartReceiver(pAdapter);
    if (EC_E_NOERROR != dwRes)
    {
        dwRetVal = dwRes;
        goto Exit;
    }
    */

    /* startup the transceiver */
    /* <= change here */
    /*
    dwRes = XXXX_StartTransceiver(pAdapter);
    if (EC_E_NOERROR != dwRes)
    {
        dwRetVal = dwRes;
        goto Exit;
    }
    */
    
    /* enqueue Adapter */
    ListAddOI(pAdapter);

    /* no errors */
    dwRetVal = EC_E_NOERROR; 
    
    /* increment instance counter */
    S_nOpenedInstances++;
    
    /* return "handle" */
    *ppvInstance = pAdapter;
    
Exit:
    if ((EC_E_NOERROR != dwRetVal) && (EC_E_INVALIDSTATE != dwRetVal))
    {
        *ppvInstance = EC_NULL;
        
        if (EC_NULL != pAdapter)
        {
            EcLinkClose(pAdapter);
            pAdapter = EC_NULL;
        }
    }
    
    return dwRetVal;
}


/********************************************************************************/
/** \brief Close Link Layer connection.
*
* \return EC_E_NOERROR or error code.
*/
static EC_T_DWORD EcLinkClose(
    EC_T_VOID* pvInstance         /* [in] instance handle */
)
{
    EC_T_DWORD          dwRetVal    = EC_E_ERROR;
    PT_DUMMY_INTERNAL   pAdapter    = (PT_DUMMY_INTERNAL)pvInstance;

    /* check for Type Signature */
    if ((EC_NULL == pAdapter) || (pAdapter->dwSignature != EC_LINK_PARMS_SIGNATURE_DUMMY))
    {
        dwRetVal = EC_E_INVALIDPARM;
        goto Exit;
    }
        
    if (EcLinkMode_INTERRUPT == pAdapter->oInitParms.linkParms.eLinkMode)
    {
        /* <= change here */
        /* i.e. Disable and remove interrupt handler */
    };
    
    if (NULL != pAdapter)
    {
        /* stop adapter */
        /* <= change here */
        /*
        XXXX_StopReceiver(pAdapter);
        XXXX_StopTransceiver(pAdapter);
        */

        /* reset adapter */
        /* <= change here */
        /*
        XXXX_Reset(pAdapter);
        */

        /* Free memory used for all descriptors */
        /* <= change here */

        /* Unmap memory */
        /* <= change here */
        /* Example */
        UnmapMemory(pAdapter);

        /* Example */
        /* Free internal memory allocated in EcLinkOpen() */
        LinkOsFree(pAdapter->pbyIntBuffer);

        /* release context */
        LinkOsReleaseContext(pAdapter->pvLinkOsContext);
        
        /* Remove adapter from the list */
        ListRmOI(pAdapter);
    
        /* Decrease counter */
        S_nOpenedInstances--;
    
        /* Free memory */
        LinkOsFree(pAdapter);
    }
    
	dwRetVal = EC_E_NOERROR;

Exit:
    return dwRetVal;
}


/********************************************************************************/
/** \brief Determine link speed.
*
* \return link speed in MBit/sec
*/
static EC_T_DWORD EcLinkGetSpeed(EC_T_VOID* pvInstance)
{
    PT_DUMMY_INTERNAL       pAdapter    = (PT_DUMMY_INTERNAL)pvInstance;
    EC_T_DWORD              dwSpeed     = 0;
    EC_T_BYTE               byPhySpeed  = 0;

    /* check for Type Signature */
    
    if ((EC_NULL == pAdapter) || (pAdapter->dwSignature != EC_LINK_PARMS_SIGNATURE_DUMMY))
    {
        dwSpeed = EC_E_INVALIDPARM;
        goto Exit;
    }

    /* get speed from HW */
    /* <= change here */
    /* Example
    dwRes = XXXX_GetPhySpeed(pAdapter, &byPhySpeed);
    if (EC_E_NOERROR != dwRes)
    {
        dwSpeed = dwRes;
        goto Exit;
    }
    */

    /* mask out unnecessary */
    switch (byPhySpeed)
    {
    case 0:     dwSpeed = EC_E_INVALIDDATA; break;
    case 1:     dwSpeed = 10;               break;
    case 2:     dwSpeed = 100;              break;
    case 3:     dwSpeed = 1000;             break;
    default:    dwSpeed = EC_E_INVALIDDATA; break;
    }

Exit:
    return dwSpeed;
}


/********************************************************************************/
/** \brief Determine link mode
*
* \return link mode
*/
static EC_T_LINKMODE EcLinkGetMode(EC_T_VOID* pvInstance)
{
    PT_DUMMY_INTERNAL       pAdapter    = (PT_DUMMY_INTERNAL)pvInstance;
    EC_T_LINKMODE           oLink       = EcLinkMode_UNDEFINED;
    
    /* check for Type Signature */
    if ((EC_NULL == pAdapter) || (pAdapter->dwSignature != EC_LINK_PARMS_SIGNATURE_DUMMY))
    {
        goto Exit;
    }
    
    oLink = pAdapter->oInitParms.linkParms.eLinkMode;

Exit:
    return oLink;
}

/********************************************************************************/
/** \brief Determine current link status.
*
* \return Current link status.
*/
static EC_T_LINKSTATUS EcLinkGetStatus(EC_T_VOID* pvInstance)
{
    PT_DUMMY_INTERNAL   pAdapter    = (PT_DUMMY_INTERNAL)pvInstance;
    EC_T_LINKSTATUS     oStatus     = eLinkStatus_UNDEFINED;
    
    /* check for Type Signature */
    if ((EC_NULL == pAdapter) || (pAdapter->dwSignature != EC_LINK_PARMS_SIGNATURE_DUMMY))
    {
        goto Exit;
    }

    /* get link status from Hardware */
    /* <= change here */
    /*
    if (EC_E_NOERROR != XXXX_GetStatusReg(pAdapter, &statusreg))
    {
        goto Exit;
    }
    */

    /* <= change here */
    /*
    oStatus = eLinkStatus_HALFDUPLEX;
    oStatus = eLinkStatus_DISCONNECTED;
    */

    /* Example */
    oStatus = eLinkStatus_OK;

Exit:

    return oStatus;
}


/********************************************************************************/
/** \brief Determine link layer MAC address
*
* \return EC_E_NOERROR or error code.
*/
static EC_T_DWORD EcLinkGetEthernetAddress(
    EC_T_VOID* pvInstance,
    EC_T_BYTE* pbyEthernetAddress   /* [out]  Ethernet MAC address */
    )
{
    PT_DUMMY_INTERNAL   pAdapter    = (PT_DUMMY_INTERNAL)pvInstance;
    EC_T_DWORD          dwRetVal    = EC_E_ERROR;
    EC_T_BYTE           abyMac[6]   = {00,01,02,03,04,05};

    /* check for parameters */
    if ((EC_NULL == pvInstance) || (EC_NULL == pbyEthernetAddress))
    {
        dwRetVal = EC_E_INVALIDPARM;
        goto Exit;
    }

    /* check for Type Signature */
    if ((EC_NULL == pAdapter) || (pAdapter->dwSignature != EC_LINK_PARMS_SIGNATURE_DUMMY))
    {
        dwRetVal = EC_E_INVALIDPARM;
        goto Exit;
    }

    /* evaluate MAC address and return it */
    /* <= change here */
    
    LinkOsMemcpy(pbyEthernetAddress, abyMac, 6);
    dwRetVal = EC_E_NOERROR;

Exit:
    return dwRetVal;
}

/********************************************************************************/
/** \brief Allocate a frame buffer used for send 
*
*    If the link layer doesn't support frame allocation, this function must return
*    EC_E_NOTSUPPORTED
*
* \return EC_E_NOERROR or error code.
*/
static EC_T_DWORD EcLinkAllocSendFrame(
    EC_T_VOID*           pvInstance,
    EC_T_LINK_FRAMEDESC* pLinkFrameDesc,     /* [in/out]  link frame descriptor */
    EC_T_DWORD           dwSize              /* [in]      size of the frame to allocate */
)
{
    EC_T_DWORD          dwRetVal    = EC_E_ERROR;
    PT_DUMMY_INTERNAL   pAdapter    = (PT_DUMMY_INTERNAL)pvInstance;

    /* check for Type Signature */
    if ((EC_NULL == pAdapter) || (pAdapter->dwSignature != EC_LINK_PARMS_SIGNATURE_DUMMY))
    {
        dwRetVal = EC_E_INVALIDPARM;
        goto Exit;
    }
    
    if (eLinkStatus_DISCONNECTED == EcLinkGetStatus(pvInstance))
    {
        dwRetVal = EC_E_INVALIDSTATE;
        goto Exit;
    }

    /* estimate how much DMA memory to allocate, reserve and return it */
    /* <= change here */

    /* Example */
    pLinkFrameDesc->pbyFrame = (EC_T_BYTE*)LinkOsMalloc(dwSize);
    pLinkFrameDesc->dwSize = dwSize;
    
    if (EC_NULL == pLinkFrameDesc->pbyFrame)
    {
        pLinkFrameDesc->dwSize = 0;
        
        dwRetVal = EC_E_NOMEMORY;
        goto Exit;
    }

    dwRetVal = EC_E_NOERROR;
Exit:
    
    return dwRetVal;
}

/*****************************************************************************/
/** \brief Send data frame
*
* \return EC_E_NOERROR or error code.
*/
static EC_T_DWORD EcLinkSendFrame(
    EC_T_VOID*           pvInstance,
    EC_T_LINK_FRAMEDESC* pLinkFrameDesc      /* [in]  link frame descriptor */
)
{
    EC_T_DWORD              dwRetVal    = EC_E_ERROR;
    PT_DUMMY_INTERNAL       pAdapter    = (PT_DUMMY_INTERNAL)pvInstance;
    
    if ((EC_NULL == pAdapter) || (pAdapter->dwSignature != EC_LINK_PARMS_SIGNATURE_DUMMY))
    {
        dwRetVal = EC_E_INVALIDPARM;
        goto Exit;
    }
    
    /* transmit Buffer. If alloc supported, check if give descriptor was allocated previously */
    /* <= change here */

    /* now really send */
    /* <= change here */

    /* Example */
    LinkOsMemcpy(pAdapter->pbyIntBuffer, pLinkFrameDesc->pbyFrame, pLinkFrameDesc->dwSize);
    pAdapter->dwFrameLen = pLinkFrameDesc->dwSize;
    pAdapter->bNewData = EC_TRUE;
    
    /* immediate after sending packet do timestamping */
    if (EC_NULL != pLinkFrameDesc->pfnTimeStamp)
    {
        *pLinkFrameDesc->pdwLastTSResult    =  pLinkFrameDesc->pfnTimeStamp(pLinkFrameDesc->pvTimeStampCtxt, pLinkFrameDesc->pdwTimeStampLo);
        *pLinkFrameDesc->pdwTimeStampPostLo = *pLinkFrameDesc->pdwTimeStampLo;
    }
    /* no errors */
    dwRetVal = EC_E_NOERROR;

Exit:

    return dwRetVal;
}

/********************************************************************************/
/** \brief Send data frame and free the frame buffer. This function must be 
*          supported if EcLinkAllocSendFrame() is supported
*
* \return EC_E_NOERROR or error code.
*/
static EC_T_DWORD EcLinkSendAndFreeFrame(
    EC_T_VOID*           pvInstance,
    EC_T_LINK_FRAMEDESC* pLinkFrameDesc      /* [in]  link frame descriptor */
)
{
    EC_T_DWORD dwRetVal = EC_E_ERROR;

    /* <= change here */

    /* Example */
    /* Send frame */
    dwRetVal = EcLinkSendFrame(pvInstance, pLinkFrameDesc);
    /* Free frame memory, no error check in this case */
    EcLinkFreeSendFrame(pvInstance, pLinkFrameDesc);

    /* nothing to do */
    return dwRetVal;
}


/********************************************************************************/
/** \brief Poll for received frame. This function is called by the ethercat Master 
*          if the function EcLinkGetMode() returns EcLinkMode_POLLING
*
* \return EC_E_NOERROR or error code.
*/
static EC_T_DWORD EcLinkRecvFrame(
    EC_T_VOID*           pvInstance,
    EC_T_LINK_FRAMEDESC* pLinkFrameDesc      /* [in]  link frame descriptor */
)
{
    EC_T_DWORD              dwRetVal    = EC_E_ERROR;
    PT_DUMMY_INTERNAL       pAdapter    = (PT_DUMMY_INTERNAL)pvInstance;

    /* check for Type Signature */
    if ((EC_NULL == pAdapter) || (pAdapter->dwSignature != EC_LINK_PARMS_SIGNATURE_DUMMY))
    {
        dwRetVal = EC_E_INVALIDPARM;
        goto Exit;
    }
    
    if (EcLinkMode_INTERRUPT == pAdapter->oInitParms.linkParms.eLinkMode)
    {
        dwRetVal = EC_E_NOTSUPPORTED;
        goto Exit;
    }
    
    if (EC_NULL == pLinkFrameDesc)
    {
        dwRetVal = EC_E_INVALIDPARM;
        goto Exit;
    }

    /* now rx single frame */
    /* <= change here */

    /* Example */
    /* Just an echo of the last frame sent */
    pLinkFrameDesc->dwSize = 0;
    if (pAdapter->bNewData)
    {
        pLinkFrameDesc->pbyFrame = pAdapter->pbyIntBuffer;
        pLinkFrameDesc->pbyFrame[17]++;
        pLinkFrameDesc->dwSize = pAdapter->dwFrameLen;
        pAdapter->bNewData = EC_FALSE;
    }

    dwRetVal = EC_E_NOERROR;

Exit:
    return dwRetVal;
}


/********************************************************************************/
/** \brief Release a frame buffer previously allocated with EcLinkAllocFrame()
*
* \return EC_E_NOERROR or error code.
*/
static EC_T_VOID  EcLinkFreeSendFrame(
    EC_T_VOID*           pvInstance,
    EC_T_LINK_FRAMEDESC* pLinkFrameDesc      /* [in]  link frame descriptor */
)
{
    PT_DUMMY_INTERNAL       pAdapter    = (PT_DUMMY_INTERNAL)pvInstance;
    
    if ((EC_NULL == pAdapter) || (pAdapter->dwSignature != EC_LINK_PARMS_SIGNATURE_DUMMY))
    {
        goto Exit;
    }
    
    /* if given descriptor was allocated correctly and is not in use by Hardware free it */
    /* <= change here */

    /* Example */
    LinkOsFree(pLinkFrameDesc->pbyFrame);
    pLinkFrameDesc->pbyFrame = EC_NULL;
    pLinkFrameDesc->dwSize = 0;
   
    EC_UNREFPARM(pLinkFrameDesc);
Exit:
    return;
}


/********************************************************************************/
/** \brief Release a frame buffer given to the ethercat master through the receive
*          callback function
*
* \return EC_E_NOERROR or error code.
*/
static EC_T_VOID  EcLinkFreeRecvFrame(
    EC_T_VOID*           pvInstance,
    EC_T_LINK_FRAMEDESC* pLinkFrameDesc      /* [in]  link frame descriptor */
)
{
    PT_DUMMY_INTERNAL   pAdapter = (PT_DUMMY_INTERNAL)pvInstance;

    /* check for Type Signature */
    if ((EC_NULL == pAdapter) || (pAdapter->dwSignature != EC_LINK_PARMS_SIGNATURE_DUMMY))
    {
        goto Exit;
    }
    
    if (EC_NULL == pLinkFrameDesc)
    {
        goto Exit;
    }

    /* Free previously received Frame Descriptor */
    /* <= change here */
    /*XXXX_FreeRecvFrame(pAdapter, pLinkFrameDesc);*/

    /* Example */
    LinkOsDbgAssert(pLinkFrameDesc->pbyFrame == pAdapter->pbyIntBuffer);

Exit:
    return;
}

/********************************************************************************/
/** \brief Multi Purpose LinkLayer IOCTL
*
* \return EC_E_NOERROR or error code.
*/

static EC_T_DWORD EcLinkIoctl(
    EC_T_VOID* pvInstance, 
    EC_T_DWORD dwCode, 
    EC_T_LINK_IOCTLPARMS*  pParms 
    )
{
    EC_T_DWORD                  dwRetVal = EC_E_ERROR;
    PT_DUMMY_INTERNAL           pAdapter = (PT_DUMMY_INTERNAL)pvInstance;
    EC_T_LINK_FRM_RECV_CLBK*    pFrameRecvClbk = { 0 };

    /* check for Type Signature */
    if ((EC_NULL == pAdapter) || (pAdapter->dwSignature != EC_LINK_PARMS_SIGNATURE_DUMMY))
    {
        goto Exit;
    }

    /* fan out IOCTL functions */

    switch (dwCode)
    {
    case EC_LINKIOCTL_REGISTER_FRAME_CLBK:

        if ((EC_NULL == pParms) || (EC_NULL == pParms->pbyInBuf)
            || (sizeof(EC_T_LINK_FRM_RECV_CLBK) != pParms->dwInBufSize))
        {
            dwRetVal = EC_E_INVALIDPARM;
            goto Exit;
        }

        pFrameRecvClbk = (EC_T_LINK_FRM_RECV_CLBK*)pParms->pbyInBuf;

        pAdapter->pvCallbackContext = pFrameRecvClbk->pvDevice;
        pAdapter->pfReceiveFrameCallback = (EC_T_RECEIVEFRAMECALLBACK)pFrameRecvClbk->pfFrameReceiveCallback;
        dwRetVal = EC_E_NOERROR;
        break;

    case EC_LINKIOCTL_UNREGISTER_FRAME_CLBK:
        pAdapter->pvCallbackContext = EC_NULL;
        pAdapter->pfReceiveFrameCallback = EC_NULL;
        dwRetVal = EC_E_NOERROR;
        break;

    case EC_LINKIOCTL_GET_ETHERNET_ADDRESS:
        if ((EC_NULL == pParms) || (EC_NULL == pParms->pbyOutBuf) || (pParms->dwOutBufSize < 6))
        {
            dwRetVal = EC_E_INVALIDPARM;
            goto Exit;
        }

        dwRetVal = EcLinkGetEthernetAddress(pAdapter, pParms->pbyOutBuf);
        break;

    default:
        dwRetVal = EC_E_NOTSUPPORTED;
        goto Exit;
        break;
    }

    /* no error */    
    dwRetVal = EC_E_NOERROR;
    
Exit:
    return dwRetVal;
}

/********************************************************************************/
/** \brief Register link layer driver.
*
* \return EC_E_NOERROR or error code.
*/
ATEMLL_API EC_T_DWORD emllRegisterDummy(
    EC_T_LINK_DRV_DESC* pLinkDrvDesc        /* [in,out] link layer driver descriptor */
   ,EC_T_DWORD          dwLinkDrvDescSize   /* [in]     size in bytes of link layer driver descriptor */
    )
{
#if (defined INCLUDE_LOG_MESSAGES)
#undef  pEcLogParmsLL
#define pEcLogParmsLL pEcLogParmsLLRegister
#endif
    EC_T_DWORD  dwResult = EC_E_NOERROR;
    EC_T_BOOL   bInterfaceSupported = EC_FALSE;

    if (pLinkDrvDesc->dwValidationPattern != LINK_LAYER_DRV_DESC_PATTERN)
    {
		EcLogMsg(EC_LOG_LEVEL_ERROR, (pEcLogContext, EC_LOG_LEVEL_ERROR, "emllRegisterDummy: invalid drive descriptor pattern 0x%x instead of 0x%x\n",
                  pLinkDrvDesc->dwValidationPattern, LINK_LAYER_DRV_DESC_PATTERN));
        dwResult = EC_E_INVALIDPARM;
        goto Exit;
    }

    /* Check the size of the given link layer driver descriptor */
    if (dwLinkDrvDescSize > sizeof(EC_T_LINK_DRV_DESC))
    {
		EcLogMsg(EC_LOG_LEVEL_ERROR, (pEcLogContext, EC_LOG_LEVEL_ERROR, "emllRegisterDummy: link layer driver descriptor size too large\n"));
        dwResult = EC_E_INVALIDPARM;
        goto Exit;
    }

    bInterfaceSupported = EC_FALSE;

    /* Check if the version of the interface is supported  */
    if ((sizeof(EC_T_LINK_DRV_DESC) == dwLinkDrvDescSize)
        && (pLinkDrvDesc->dwInterfaceVersion == LINK_LAYER_DRV_DESC_VERSION))
    {
        bInterfaceSupported = EC_TRUE;
    }

    /* Interface is not supported. */
    if (!bInterfaceSupported)
    {
		EcLogMsg(EC_LOG_LEVEL_ERROR, (pEcLogContext, EC_LOG_LEVEL_ERROR, "emllRegisterDummy: invalid drive descriptor interface version 0x%x instead of 0x%x\n",
            pLinkDrvDesc->dwInterfaceVersion, LINK_LAYER_DRV_DESC_VERSION));
        dwResult = EC_E_INVALIDPARM;
        goto Exit;
    }

    pLinkDrvDesc->pfEcLinkOpen = EcLinkOpen;
    pLinkDrvDesc->pfEcLinkClose = EcLinkClose;
    pLinkDrvDesc->pfEcLinkSendFrame = EcLinkSendFrame;
    pLinkDrvDesc->pfEcLinkSendAndFreeFrame = EcLinkSendAndFreeFrame;
    pLinkDrvDesc->pfEcLinkRecvFrame = EcLinkRecvFrame;
    pLinkDrvDesc->pfEcLinkAllocSendFrame = EcLinkAllocSendFrame;
    pLinkDrvDesc->pfEcLinkFreeSendFrame  = EcLinkFreeSendFrame ;
    pLinkDrvDesc->pfEcLinkFreeRecvFrame  = EcLinkFreeRecvFrame ;
    pLinkDrvDesc->pfEcLinkGetEthernetAddress = EcLinkGetEthernetAddress;
    pLinkDrvDesc->pfEcLinkGetStatus = EcLinkGetStatus;
    pLinkDrvDesc->pfEcLinkGetSpeed = EcLinkGetSpeed;
    pLinkDrvDesc->pfEcLinkGetMode = EcLinkGetMode;
	pLinkDrvDesc->pfEcLinkIoctl = EcLinkIoctl;

Exit:
    return dwResult;

#if (defined INCLUDE_LOG_MESSAGES)
#undef  pEcLogParmsLL
#define pEcLogParmsLL pEcLogParmsLLDefault
#endif
}

/*-END OF SOURCE FILE--------------------------------------------------------*/
