/*-----------------------------------------------------------------------------
 * Copyright                acontis technologies GmbH, Weingarten, Germany
 * Description              Link Layer Test program
 * Response                 mdu
 *---------------------------------------------------------------------------*/

/*-INCLUDES------------------------------------------------------------------*/
#include "llAccept.h"

#include "EcOs.h"
#include "EcLinkDummy.h"
#include "EcTimer.h"

#if (defined VXWORKS)
#include <intLib.h>
#endif

#ifdef UNDER_CE
#include "pkfuncs.h"
#include <iltiming.h>
#include "nkintr.h"
#endif

/*-DEFINES-------------------------------------------------------------------*/
#define MAX_FRAME_SIZE 1416

/*-TYPEDEFS------------------------------------------------------------------*/

/*-GLOBAL VARIABLES-----------------------------------------------------------*/

/*-LOCAL VARIABLES-----------------------------------------------------------*/
extern volatile EC_T_BOOL    bRun;

static EC_T_DWORD          S_dwIrqFrmCountRx = 0;
static EC_T_BOOL           S_bIrqSeqIdxValid = EC_FALSE;
static EC_T_BYTE           S_byIrqLastSeqIdx = 0;
static EC_T_DWORD          S_dwIrqByteCountTot = 0;
static EC_T_DWORD          S_dwIrqSeqErrCount = 0;
static EC_T_DWORD          S_dwIrqFrmCountTotRx = 0;
static EC_T_DWORD          S_dwIrqByteCount = 0;

/*-FORWARD DECLARATIONS------------------------------------------------------*/
EC_T_DWORD ReceiveFrameCallback(EC_T_VOID* pvContext, EC_T_LINK_FRAMEDESC* pLinkFrameDesc, EC_T_BOOL* pbFrameProcessed);

EC_T_DWORD FullTest(
    EC_T_LINK_DRV_DESC*     pLinkDev,           /**< [in]   LinkLayer Desc */
    EC_T_VOID*              pvHandleLL,         /**< [in]   LinkLayer Handle */
    APP_LLA_MODE            eMode,              /**< [in]   Test Mode */
    EC_T_DWORD              dwDuration,         /**< [in]   Time to send */
    EC_T_DWORD              dwDataSize,         /**< [in]   Size of Data in frame */
    EC_T_VOID*              pvTimingEvent,      /**< [in]   timing event */
    EC_T_DWORD              dwBasePeriod        /**< [in]   clock rate in usec */
    );

/********************************************************************************/
/** \brief  LinkLayer Test Application.
*
* This is a LinkLayer Test application.
* 
* \return  Status value.
*/
EC_T_DWORD llAcceptRun(
    EC_T_DWORD          dwDuration,             /* [in]  test duration in msec (0 = forever) */
    EC_T_LINK_PARMS*    poLinkDevOpenParam,     /* [in]  pointer to link parameter */
    EC_T_DWORD          dwCpuIndex,             /* [in]  SMP only: CPU index */
    EC_T_INT            nVerbose,               /* [in]  Verbosity level */
    APP_LLA_MODE        eMode,                  /* [in]  Test Mode */
    EC_T_DWORD          dwDataSize,             /* [in]  Size of Data in frame */
    EC_T_VOID*          pvTimingEvent,          /* [in]  timing event */
    EC_T_DWORD          dwBasePeriod            /* [in]  clock rate in usec */
    )
{
    EC_T_DWORD          dwRetVal                    = EC_E_ERROR;
    EC_T_DWORD          dwRes                       = EC_E_ERROR;
    CEcTimer            oTimeout;    
    EC_T_CPUSET         CpuSet;

    EC_PF_LLREGISTER    pfRegLL                     = EC_NULL;
    EC_T_LINK_DRV_DESC  oLinkDrvParm;
    EC_T_PVOID          pvLLHandle                  = EC_NULL;
    EC_T_LINKSTATUS     eLnkState                   = eLinkStatus_UNDEFINED;
    EC_T_DWORD          dwLinkStatusCheckCount      = 0;

    OsMemset(&oLinkDrvParm, 0, sizeof(EC_T_LINK_DRV_DESC));

    EC_CPUSET_ZERO(CpuSet);
    EC_CPUSET_SET(CpuSet, dwCpuIndex); 
    EC_UNREFPARM(dwCpuIndex);                       /* for WinCE */
    OsSetThreadAffinity(EC_NULL, CpuSet);

#ifdef UNDER_CE
    OsSetThreadPriority(GetCurrentThread(), SEND_THREAD_PRIO);
#endif
                   
    /****************************************/
    /* static EtherCAT master configuration */
    /****************************************/
    EcLogMsg(EC_LOG_LEVEL_INFO, (pEcLogContext, EC_LOG_LEVEL_INFO, "Open LinkLayer"));
    EcLogMsg(EC_LOG_LEVEL_INFO, (pEcLogContext, EC_LOG_LEVEL_INFO, "=============="));
    
    pfRegLL = OsGetLinkLayerRegFunc(poLinkDevOpenParam->szDriverIdent);
    if (EC_NULL == pfRegLL)
    {
        EcLogMsg(EC_LOG_LEVEL_ERROR, (pEcLogContext, EC_LOG_LEVEL_ERROR, "Cannot find registration function for link layer '%s'",
            poLinkDevOpenParam->szDriverIdent));
        dwRetVal = EC_E_OPENFAILED;
        goto Exit;
    }
    
    oLinkDrvParm.dwValidationPattern    = LINK_LAYER_DRV_DESC_PATTERN;
    oLinkDrvParm.dwInterfaceVersion     = LINK_LAYER_DRV_DESC_VERSION;
    OsMemcpy(&oLinkDrvParm.LogParms, &poLinkDevOpenParam->LogParms, sizeof(EC_T_LOG_PARMS));

    dwRes = pfRegLL(&oLinkDrvParm, sizeof(EC_T_LINK_DRV_DESC));
    if (EC_E_NOERROR != dwRes)
    {
        EcLogMsg(EC_LOG_LEVEL_ERROR, (pEcLogContext, EC_LOG_LEVEL_ERROR, "Error registering LinkLayer: 0x%lx", dwRes));
        dwRetVal = dwRes;
        goto Exit;
    }

    /* now open LL */
    dwRes = oLinkDrvParm.pfEcLinkOpen(poLinkDevOpenParam, ReceiveFrameCallback, EC_NULL, EC_NULL, &pvLLHandle);
    if (EC_E_NOERROR != dwRes)
    {
        EcLogMsg(EC_LOG_LEVEL_ERROR, (pEcLogContext, EC_LOG_LEVEL_ERROR, "Error opening LinkLayer: 0x%lx", dwRes));
        dwRetVal = dwRes;
        goto Exit;
    }

#if (defined QNX6)
    if (pvTimingEvent != EC_NULL)
    {
        _qnx6_timing_event *timing_event = (_qnx6_timing_event*)pvTimingEvent;

        SIGEV_INTR_INIT(&timing_event->intr_event);

        timing_event->intr_id = InterruptAttachEvent(timing_event->intr, &timing_event->intr_event, _NTO_INTR_FLAGS_TRK_MSK | _NTO_INTR_FLAGS_END);

        if (timing_event->intr_id == -1)
        {
            EcLogMsg(EC_LOG_LEVEL_ERROR, (pEcLogContext, EC_LOG_LEVEL_ERROR, "tEcJobTask:: cannot attach auxiliary clock interrupt event: %i", errno));
            goto Exit;
        }
    }
#endif /* QNX6 */


    /* Before start wait for link up. */
    do 
    {
        OsSleep(10);
        
        eLnkState = oLinkDrvParm.pfEcLinkGetStatus(pvLLHandle);
        dwLinkStatusCheckCount++;
    }
    while ((eLinkStatus_OK != eLnkState) && (dwLinkStatusCheckCount < 1000));
    
    /* Check if link is in expected state */
    if (eLinkStatus_OK != eLnkState)
    {
        EcLogMsg(EC_LOG_LEVEL_ERROR, (pEcLogContext, EC_LOG_LEVEL_ERROR, "Link is not available with 100 MBit/s full duplex!\nPlease check cables and run application again."));
        /* Link is not available with 100 MBit/s full duplex. */
        dwRetVal = EC_E_LINK_DISCONNECTED;
        goto Exit;        
    }
    /* print MAC address */
    {
        EC_T_BYTE aSrcMacAddress[6];
        OsMemset(aSrcMacAddress, 0, sizeof(aSrcMacAddress));

        dwRes = oLinkDrvParm.pfEcLinkGetEthernetAddress(pvLLHandle, aSrcMacAddress);
        if (dwRes != EC_E_NOERROR)
        {
            EcLogMsg(EC_LOG_LEVEL_ERROR, (pEcLogContext, EC_LOG_LEVEL_ERROR, "Cannot get MAC address: 0x%lx)", dwRes));
        }
        EcLogMsg(EC_LOG_LEVEL_INFO, (pEcLogContext, EC_LOG_LEVEL_INFO, "Network adapter MAC: %02X-%02X-%02X-%02X-%02X-%02X",
            aSrcMacAddress[0], aSrcMacAddress[1], aSrcMacAddress[2], aSrcMacAddress[3], aSrcMacAddress[4], aSrcMacAddress[5]));
    }

    switch (eMode)
    {
    case allam_tx:
        EcLogMsg(EC_LOG_LEVEL_INFO, (pEcLogContext, EC_LOG_LEVEL_INFO, "Sending with period %d usec", dwBasePeriod));
        break;
    case allam_rx:
        EcLogMsg(EC_LOG_LEVEL_INFO, (pEcLogContext, EC_LOG_LEVEL_INFO, "Receiving with period %d usec", dwBasePeriod));
        break;
    case allam_trx:
        EcLogMsg(EC_LOG_LEVEL_INFO, (pEcLogContext, EC_LOG_LEVEL_INFO, "Sending/Receiving with period %d usec", dwBasePeriod));
        break;
    case allam_txf:
        EcLogMsg(EC_LOG_LEVEL_INFO, (pEcLogContext, EC_LOG_LEVEL_INFO, "Sending with full speed"));
        break;
    case allam_rxf:
        EcLogMsg(EC_LOG_LEVEL_INFO, (pEcLogContext, EC_LOG_LEVEL_INFO, "Receiving with full speed"));
        break;
    case allam_trxf:
        EcLogMsg(EC_LOG_LEVEL_INFO, (pEcLogContext, EC_LOG_LEVEL_INFO, "Sending/Receiving with full speed"));
        break;
    case allam_irq:
        EcLogMsg(EC_LOG_LEVEL_INFO, (pEcLogContext, EC_LOG_LEVEL_INFO, "Sending/Receiving in interrupt mode"));
        break;
    default:
        EcLogMsg(EC_LOG_LEVEL_ERROR, (pEcLogContext, EC_LOG_LEVEL_ERROR, "Unknown test mode!"));
        OsDbgAssert(EC_FALSE);
        break;
    }

    if ((eMode == allam_tx) || (eMode == allam_rx) || (eMode == allam_trx))
    {
        FullTest(&oLinkDrvParm, pvLLHandle, eMode, dwDuration, dwDataSize, pvTimingEvent, dwBasePeriod);
    }
    else if ((eMode == allam_txf) || (eMode == allam_rxf) || (eMode == allam_trxf) || (eMode == allam_irq))
    {
        FullTest(&oLinkDrvParm, pvLLHandle, eMode, dwDuration, dwDataSize, pvTimingEvent, 0);
    }

    dwRetVal = EC_E_NOERROR;

Exit:
    EcLogMsg(EC_LOG_LEVEL_INFO, (pEcLogContext, EC_LOG_LEVEL_INFO, "\nClose LinkLayer"));
    EcLogMsg(EC_LOG_LEVEL_INFO, (pEcLogContext, EC_LOG_LEVEL_INFO, "==============="));

    /* release resources */
    if ((EC_NULL != pvLLHandle) && (EC_NULL != oLinkDrvParm.pfEcLinkClose))
    {
        oLinkDrvParm.pfEcLinkClose(pvLLHandle);
        pvLLHandle = EC_NULL;
    }

    return dwRetVal;
}

/********************************************************************************/
/** \brief This function is called by the link layer after a new frame was received.
*
* \return status code
*/
EC_T_DWORD ReceiveFrameCallback(EC_T_VOID* pvContext, EC_T_LINK_FRAMEDESC* pLinkFrameDesc, EC_T_BOOL* pbFrameProcessed)
{
    EC_T_DWORD dwRetVal = EC_E_NOERROR;

    EC_UNREFPARM(pvContext);
    if ((0 == pLinkFrameDesc->dwSize) || (EC_NULL == pLinkFrameDesc->pbyFrame))
    {
        /* nothing received */
    }
    else
    {
        /* did receive something */

        /* check for sequence errors */
        if (S_bIrqSeqIdxValid)
        {
            if ((EC_T_BYTE)(S_byIrqLastSeqIdx + 1) != pLinkFrameDesc->pbyFrame[17])
            {
                S_dwIrqSeqErrCount++;
            }

            S_byIrqLastSeqIdx = pLinkFrameDesc->pbyFrame[17];
        }
        else
        {
            S_byIrqLastSeqIdx = pLinkFrameDesc->pbyFrame[17];
            S_bIrqSeqIdxValid = EC_TRUE;
        }

        /* bump counters */
        S_dwIrqFrmCountRx++;
        S_dwIrqFrmCountTotRx++;

        S_dwIrqByteCount += pLinkFrameDesc->dwSize;
        S_dwIrqByteCountTot += pLinkFrameDesc->dwSize;
    }

    /* frame is processed! */
    *pbFrameProcessed = EC_TRUE;

    return dwRetVal;
}

/***************************************************************************************************/
/**
\brief  Wait until next cycle starts

\return EC_E_NOERROR.
*/
void DelayUntilNextCycle(EC_T_VOID* pvTimingEvent, EC_T_DWORD dwBasePeriod)
{
#if (defined RTAI) || (defined XENOMAI)
   OsSleepTillTick(); /* period is set after creating jobtask */
#else
   OsWaitForEvent(pvTimingEvent, EC_WAITINFINITE);
#endif /* !RTAI && !XENOMAI*/
}

/***************************************************************************************************/
/**
\brief  Full speed TX RX.

Send as fast as possible and receive.
\return EC_E_NOERROR.
*/
EC_T_DWORD FullTest(
    EC_T_LINK_DRV_DESC*     pLinkDev,           /**< [in]   LinkLayer Desc */
    EC_T_VOID*              pvHandleLL,         /**< [in]   LinkLayer Handle */
    APP_LLA_MODE            eMode,              /**< [in]   Test Mode */
    EC_T_DWORD              dwDuration,         /**< [in]   Time to send */
    EC_T_DWORD              dwDataSize,         /**< [in]   Size of Data in frame */
    EC_T_VOID*              pvTimingEvent,      /**< [in]   timing event */
    EC_T_DWORD              dwBasePeriod        /**< [in]   clock rate in usec */
    )
{
    EC_T_DWORD          dwRetVal        = EC_E_ERROR;
    EC_T_BOOL           bAllocSupport   = EC_TRUE;
    EC_T_DWORD          dwRes           = EC_E_ERROR;

    EC_T_LINK_FRAMEDESC oFrameTx        = {0};
    EC_T_DWORD          dwFrmCountTx    = 0;
    EC_T_DWORD          dwFrmCountTotTx = 0;

    EC_T_LINK_FRAMEDESC oFrameRx        = {0};
    EC_T_DWORD          dwFrmCountRx    = 0;
    EC_T_DWORD          dwFrmCountTotRx = 0;

    CEcTimer            oTimePrint;
    CEcTimer            oTimeOut;

    EC_T_DWORD          dwMsecStart     = 0;
    EC_T_DWORD          dwMsecLast      = 0;
    EC_T_DWORD          dwMsecCur       = 0;

    EC_T_DWORD          dwNoCable       = 0;
    EC_T_DWORD          dwEcFrmLen      = 0;
    EC_T_DWORD          dwEcDgrmLen     = 0;
    EC_T_BYTE           byIndex         = 0;

    EC_T_BOOL           bSeqIdxValid    = EC_FALSE;
    EC_T_BYTE           byLastSeqIdx    = 0;
    EC_T_DWORD          dwSeqErrCount   = 0;
    EC_T_DWORD          dwByteCount     = 0;
    EC_T_DWORD          dwByteCountTot  = 0;
    
    EC_T_BYTE           abyProtoFrame[] = 
    {
        0xff, 0xff, 0xff, 0xff, 0xff, 0xff,     /* Broadcast Destination */                 /*  0 -  5 = 6 */ 
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00,     /* Frame Source */                          /*  6 - 11 = 6 */
        0x88, 0xa4,                             /* EtherType = Ecat */                      /* 12 - 13 = 2 */
        0xda, 0x15,                             /* E88A4 Length 0x0e, Type ECAT (0x1) */    /* 14 - 15 = 2 */
        0x01,                                   /* APRD */                                  /* 16      = 1 */
        0x00,                                   /* Index */                                 /* 17      = 1 */
        0x00, 0x00,                             /* Slave Address (ADP) */                   /* 18 - 19 = 2 */
        0x00, 0x00,                             /* Offset Address (ADO) */                  /* 20 - 21 = 2 */
        0x88, 0x05,                             /* Length - Last sub command */             /* 22 - 23 = 2 */
    };

    pLinkDev->pfEcLinkGetEthernetAddress(pvHandleLL, &abyProtoFrame[6]);

    /* frame size needs to be at least header + WKC */
    if (dwDataSize > MAX_FRAME_SIZE)
    {
        EcLogMsg(EC_LOG_LEVEL_ERROR, (pEcLogContext, EC_LOG_LEVEL_ERROR, "Frame size at max %d Bytes, corrected", MAX_FRAME_SIZE));
        dwDataSize = MAX_FRAME_SIZE;
    }

    /* configure counters / length fields */
    /* frame Length */
    dwEcFrmLen = 12;   /* MAC addrs */
    dwEcFrmLen += 2;   /* Protocoll */
    dwEcFrmLen += 2;   /* E88a4 len / type */
    dwEcFrmLen += 2;   /* Cmd/Idx */
    dwEcFrmLen += 4;   /* Slave Addr */
    dwEcFrmLen += 2;   /* Len/Next */
    dwEcFrmLen += 2;   /* IRQ */
    dwEcFrmLen += 2;   /* WKC */
    dwEcFrmLen += dwDataSize;

    // | Length 11Bit | Res 1Bit | Type 4 Bit |
    EC_SET_FRM_WORD(&(abyProtoFrame[14]), EC_LOWORD((dwEcFrmLen - 2) | 0x1000));

    /* dgram length */
    dwEcDgrmLen = dwDataSize;
    EC_SET_FRM_WORD(&(abyProtoFrame[22]), EC_LOWORD(dwEcDgrmLen));

    oTimeOut.Start(dwDuration);
    oTimePrint.Start(1000);

    dwMsecLast = dwMsecStart = OsQueryMsecCount();

    do
    {
        if ((eMode == allam_tx) || (eMode == allam_txf) || (eMode == allam_trx) || (eMode == allam_trxf) || (eMode == allam_irq))
        {
            /***** Send frame *****/

            /* alloc frame */
            if (bAllocSupport)
            {
                dwRes = pLinkDev->pfEcLinkAllocSendFrame(pvHandleLL, &oFrameTx, EC_MAX(dwEcFrmLen, 60));
                if (EC_E_NOTSUPPORTED == dwRes)
                {
                    bAllocSupport = EC_FALSE;
                    oFrameTx.dwSize = EC_MAX(dwEcFrmLen, 60);
                    oFrameTx.pbyFrame = (EC_T_BYTE*)OsMalloc(oFrameTx.dwSize);
                    if (EC_NULL == oFrameTx.pbyFrame)
                    {
                        EcLogMsg(EC_LOG_LEVEL_ERROR, (pEcLogContext, EC_LOG_LEVEL_ERROR, "Cannot create frame buffer"));
                        dwRetVal = EC_E_NOMEMORY;
                        goto Exit;
                    }

                    OsMemset(oFrameTx.pbyFrame, 0, oFrameTx.dwSize);
                }
                else if (EC_E_NOMEMORY == dwRes)
                {
                    /* ignore */
                }
                else if (EC_E_NOERROR == dwRes)
                {
                    /* everything is fine :) */
                }
                else
                {
                    EcLogMsg(EC_LOG_LEVEL_ERROR, (pEcLogContext, EC_LOG_LEVEL_ERROR, "Cannot alloc send frame, error: 0x%lx", dwRes));
                    dwRetVal = dwRes;
                    goto Exit;
                }
            }

            if ((EC_NULL != oFrameTx.pbyFrame) && (0 != oFrameTx.dwSize))
            {
                /* fill frame Headers */
                OsMemcpy(oFrameTx.pbyFrame, abyProtoFrame, sizeof(abyProtoFrame));

                /* increment cmd,idx field (idx) */
                oFrameTx.pbyFrame[17] = byIndex++;

                /* send frame */
                if (bAllocSupport)
                {
                    dwRes = pLinkDev->pfEcLinkSendAndFreeFrame(pvHandleLL, &oFrameTx);
                }
                else
                {
                    dwRes = pLinkDev->pfEcLinkSendFrame(pvHandleLL, &oFrameTx);
                }

                if (EC_E_NOERROR == dwRes)
                {
                    dwFrmCountTx++;
                    dwFrmCountTotTx++;
                    dwNoCable = 0;
                }
                else if (EC_E_LINK_DISCONNECTED == dwRes)
                {
                    byIndex--;
                    dwNoCable++;
                    if (50 == dwNoCable)
                    {
                        EcLogMsg(EC_LOG_LEVEL_ERROR, (pEcLogContext, EC_LOG_LEVEL_ERROR, "No response from slaves, cable missing"));
                        dwNoCable = 0;
                    }
                    OsSleep(50);
                }
                else
                {
                    byIndex--;
                    if (bAllocSupport)
                    {
                        pLinkDev->pfEcLinkFreeSendFrame(pvHandleLL, &oFrameTx);
                    }
                }
            }
        }

        if ((eMode == allam_rx) || (eMode == allam_rxf) || (eMode == allam_trx) || (eMode == allam_trxf))
        {
            /***** Recieve frame *****/
            do
            {
                /* Prepare receive buffer */
                if (0 != oFrameRx.dwSize)
                {
                    OsMemset(&oFrameRx, 0, sizeof(EC_T_LINK_FRAMEDESC));
                }

                /* Receive frame */
                dwRes = pLinkDev->pfEcLinkRecvFrame(pvHandleLL, &oFrameRx);
                if (EC_E_NOERROR != dwRes)
                {
                    if ((0 != oFrameRx.dwSize) && (EC_NULL != oFrameRx.pbyFrame))
                    {
                        pLinkDev->pfEcLinkFreeRecvFrame(pvHandleLL, &oFrameRx);
                    }

                    continue;
                }

                if ((0 == oFrameRx.dwSize) || (EC_NULL == oFrameRx.pbyFrame))
                {
                    /* nothing received */
                    break;
                }
                else
                {
                    /* did receive something */

                    /* check for sequence errors */
                    if (bSeqIdxValid)
                    {
                        if ((EC_T_BYTE)(byLastSeqIdx + 1) != oFrameRx.pbyFrame[17])
                        {
                            dwSeqErrCount++;
                        }

                        byLastSeqIdx = oFrameRx.pbyFrame[17];
                    }
                    else
                    {
                        byLastSeqIdx = oFrameRx.pbyFrame[17];
                        bSeqIdxValid = EC_TRUE;
                    }

                    /* bump counters */
                    dwFrmCountRx++; dwFrmCountTotRx++;

                    dwByteCount += oFrameRx.dwSize;
                    dwByteCountTot += oFrameRx.dwSize;

                    pLinkDev->pfEcLinkFreeRecvFrame(pvHandleLL, &oFrameRx);
                }
            }
            while (bRun && !oTimeOut.IsElapsed());
        }

        /* show statistics */
        if (oTimePrint.IsElapsed())
        {
            dwMsecCur = OsQueryMsecCount();

            if ((eMode == allam_tx) || (eMode == allam_txf) || (eMode == allam_trx) || (eMode == allam_trxf) || (eMode == allam_irq))
            {
                EcLogMsg(EC_LOG_LEVEL_INFO, (pEcLogContext, EC_LOG_LEVEL_INFO, "Sent     %d frames = %ld Bytes/sec",
                    dwFrmCountTx, (EC_T_DWORD)(((double)dwFrmCountTx * dwEcFrmLen)) / ((dwMsecCur - dwMsecLast) / 1000)));
                dwFrmCountTx = 0;
            }

            if ((eMode == allam_rx) || (eMode == allam_rxf) || (eMode == allam_trx) || (eMode == allam_trxf))
            {
                EcLogMsg(EC_LOG_LEVEL_INFO, (pEcLogContext, EC_LOG_LEVEL_INFO, "Received %d frames = %ld Bytes/sec (Seq ErrCount Total : %ld)",
                    dwFrmCountRx, (EC_T_DWORD)(((double)dwByteCount) / ((dwMsecCur - dwMsecLast) / 1000)), dwSeqErrCount));
                dwFrmCountRx = 0;
            }

            dwByteCount = 0;
            dwMsecLast = dwMsecCur;
            oTimePrint.Start(1000);
        }

        if (dwBasePeriod > 0)
        {
            DelayUntilNextCycle(pvTimingEvent, dwBasePeriod);
        }
    }
    while (bRun && !oTimeOut.IsElapsed());

    if (!bAllocSupport)
    {
        OsFree(oFrameTx.pbyFrame);
    }

    /* show statistics */
    dwMsecCur = OsQueryMsecCount();

    if ((eMode == allam_tx) || (eMode == allam_txf) || (eMode == allam_trx) || (eMode == allam_trxf) || (eMode == allam_irq))
    {
        EcLogMsg(EC_LOG_LEVEL_INFO, (pEcLogContext, EC_LOG_LEVEL_INFO, "Sent total     %d Frames = %ld Bytes/sec",
            dwFrmCountTotTx, (EC_T_DWORD)(((double)dwFrmCountTotTx) * dwEcFrmLen) / ((dwMsecCur - dwMsecStart) / 1000)));
    }

    if ((eMode == allam_rx) || (eMode == allam_rxf) || (eMode == allam_trx) || (eMode == allam_trxf))
    {
        EcLogMsg(EC_LOG_LEVEL_INFO, (pEcLogContext, EC_LOG_LEVEL_INFO, "Received total %d Frames = %ld Bytes/sec (Seq ErrCount Total : %ld)",
            dwFrmCountTotRx, (EC_T_DWORD)(((double)dwByteCountTot) / ((dwMsecCur - dwMsecStart) / 1000)), dwSeqErrCount));
    }

    dwRetVal = EC_E_NOERROR;

Exit:
    return dwRetVal;
}
   
#if (defined RTOS_32) || (defined EC_VERSION_RTEMS)
/* since RTOS_32 does not support a dynamic load concept (not correctly working) we do link libs.
 * to have different LinkLayers supported, the function <OsGetLinkLayerRegFunc> is transferred from
 * the OsLayer to the application, which lets device which LinkLayer to call, without having the AtEcat.lib
 * recompiled (necc. for binary distribution)
 */
#include "eclink.h"
extern EC_T_DWORD emllRegisterI8255x(
    EC_T_LINK_DRV_DESC*     pLinkDrvDesc        /* [in,out] link layer driver descriptor */
   ,EC_T_DWORD              dwLinkDrvDescSize   /* [in]     size in bytes of link layer driver descriptor */
);

extern EC_T_DWORD emllRegisterI8254x(
    EC_T_LINK_DRV_DESC*     pLinkDrvDesc        /* [in,out] link layer driver descriptor */
   ,EC_T_DWORD              dwLinkDrvDescSize   /* [in]     size in bytes of link layer driver descriptor */
);

extern EC_T_DWORD emllRegisterRTL8139(
    EC_T_LINK_DRV_DESC*     pLinkDrvDesc        /* [in,out] link layer driver descriptor */
   ,EC_T_DWORD              dwLinkDrvDescSize   /* [in]     size in bytes of link layer driver descriptor */
);

extern EC_T_DWORD emllRegisterRTL8169(
    EC_T_LINK_DRV_DESC*     pLinkDrvDesc        /* [in,out] link layer driver descriptor */
   ,EC_T_DWORD              dwLinkDrvDescSize   /* [in]     size in bytes of link layer driver descriptor */
);

extern EC_T_DWORD emllRegisterCCAT(
    EC_T_LINK_DRV_DESC*     pLinkDrvDesc        /* [in,out] link layer driver descriptor */
   ,EC_T_DWORD              dwLinkDrvDescSize   /* [in]     size in bytes of link layer driver descriptor */
);

EC_PF_LLREGISTER OsGetLinkLayerRegFunc(EC_T_CHAR* szDriverIdent)
{
    EC_PF_LLREGISTER pfLlRegister = EC_NULL;

    if (0 == OsStrcmp(EC_LINK_PARMS_IDENT_I8254X, szDriverIdent))
    {
        pfLlRegister = (EC_PF_LLREGISTER)emllRegisterI8254x;
    }
#if (defined RTOS_32)
    else if (0 == OsStrcmp("I8255x", szDriverIdent))
    {   
        pfLlRegister = (EC_PF_LLREGISTER)emllRegisterI8255x;
    }
    else if (0 == OsStrcmp("EMRTL8319", szDriverIdent))
    {   
        pfLlRegister = (EC_PF_LLREGISTER)emllRegisterRTL8139;
    }
#endif
#if (defined EC_VERSION_RTEMS)
    else if (0 == OsStrcmp(EC_LINK_PARMS_IDENT_CCAT, szDriverIdent))
    {
    	pfLlRegister = (EC_PF_LLREGISTER)emllRegisterCCAT;
    }
#endif
#if !(defined EC_VERSION_ECOS)
    else if (0 == OsStrcmp(EC_LINK_PARMS_IDENT_RTL8169, szDriverIdent))
    {
    	pfLlRegister = (EC_PF_LLREGISTER)emllRegisterRTL8169;
    }
    else
    {
        pfLlRegister = EC_NULL;
    }
#endif
    return pfLlRegister;
}
#endif

/*-END OF SOURCE FILE--------------------------------------------------------*/
